
"""
ATL_path
========
包含了 `mkdir_or_exist()`, `find_data_list()` 创建文件夹和寻找数据集名称两大函数

用法：
----
    >>> # 在开头复制这一句
    >>> from ATL_path import mkdir_or_exist, find_data_list, scandir
    ________________________________________________________________
    >>> # 示例1-创建文件夹：
    >>> mkdir_or_exist(xxxx) # 创建文件夹, 存在则不创建
    ________________________________________________________________
    >>> # 示例2-寻找所有符合后缀文件夹名称(绝对路径):
    >>> # 寻找尾缀为'.jpg'的数据集列表
    >>> img_list = find_data_list(img_root_path: str, suffix: str ='.jpg') 
        for img_path in tqdm(img_list):
            print(img_path)
"""

import os
import os.path as osp
from pathlib import Path
from typing import List

def check_file_exist(filename, msg_tmpl='file "{}" does not exist'):
    if not osp.isfile(filename):
        raise FileNotFoundError(msg_tmpl.format(filename))


def mkdir_or_exist(dir_name, mode=0o777):
    """
    若不存在，则创建文件夹，代替os.path.exist()

    示例-创建文件夹：
    >>> mkdir_or_exist(xxxx) # 创建文件夹, 存在则不创建
    """
    if dir_name == '':
        return
    dir_name = osp.expanduser(dir_name)
    os.makedirs(dir_name, mode=mode, exist_ok=True)


def symlink(src, dst, overwrite=True, **kwargs):
    if os.path.lexists(dst) and overwrite:
        os.remove(dst)
    os.symlink(src, dst, **kwargs)


def scandir(dir_path, suffix=None, recursive=False, case_sensitive=True):
    """Scan a directory to find the interested files.

    Args:
        dir_path (str | :obj:`Path`): Path of the directory.
        suffix (str | tuple(str), optional): File suffix that we are
            interested in. Default: None.
        recursive (bool, optional): If set to True, recursively scan the
            directory. Default: False.
        case_sensitive (bool, optional) : If set to False, ignore the case of
            suffix. Default: True.

    Returns:
        A generator for all the interested files with relative paths.
    """
    if isinstance(dir_path, (str, Path)):
        dir_path = str(dir_path)
    else:
        raise TypeError('"dir_path" must be a string or Path object')

    if (suffix is not None) and not isinstance(suffix, (str, tuple)):
        raise TypeError('"suffix" must be a string or tuple of strings')

    if suffix is not None and not case_sensitive:
        suffix = suffix.lower() if isinstance(suffix, str) else tuple(
            item.lower() for item in suffix)

    root = dir_path

    def _scandir(dir_path, suffix, recursive, case_sensitive):
        for entry in os.scandir(dir_path):
            if not entry.name.startswith('.') and entry.is_file():
                rel_path = osp.relpath(entry.path, root)
                _rel_path = rel_path if case_sensitive else rel_path.lower()
                if suffix is None or _rel_path.endswith(suffix):
                    yield rel_path
            elif recursive and os.path.isdir(entry.path):
                # scan recursively if entry.path is a directory
                yield from _scandir(entry.path, suffix, recursive,
                                    case_sensitive)

    return _scandir(dir_path, suffix, recursive, case_sensitive)


def find_data_list(img_root_path: str, suffix: str ='.jpg') -> List:
    
    """根据给定的数据集根目录，找出其下所有符合后缀的数据集图片完整路径

    Args:
        img_root_path (str): 数据集根目录.
        suffix (str): 后缀名. Defaults to '.jpg'.

    Returns:
        List: 所有符合后缀的数据集图片完整路径(经过sorted()排序).
    """     
    print('--------------------------------------------------------------')
    print(f'-- 正在读取数据集列表... "{img_root_path}" ')

    img_list = []
    for img_name in scandir(img_root_path, suffix=suffix, recursive=True):
        if suffix in img_name:
            img_path = os.path.join(img_root_path, img_name)
            img_list.append(img_path)
    print(f'-- 共在 "{img_root_path}" 下寻找到 *{suffix} {len(img_list)} 项')
    print('-------------------------------------------------------------')
    return sorted(img_list)


"""
ATL_Tools
========
包含了 `ATL_path`和`ATL_gdal`两大工具

用法：
----
    >>> # 在开头复制这一句
    >>> from ATL_Tools import mkdir_or_exist, find_data_list # 

    >>> from ATL_Tools.ATL_gdal import (
            read_img_to_array_with_info, # ✔读取影像为数组并返回信息
            read_img_to_array,  # ✔读取影像为数组
            save_ds_to_tif,     # ✔将GDAL dataset数据格式写入tif保存
            save_array_to_tif,  # 将数组格式写入tif保存
            read_img_get_geo,   # ✔计算影像角点的地理坐标或投影坐标
            ds_get_img_geo,     # 读取dataset格式，计算影像角点的地理坐标或投影坐标
            pix_to_geo,         # 计算影像某一像素点的地理坐标或投影坐标
            geo_to_pix,         # 根据GDAL的仿射变换参数模型将给定的投影或地理坐标转为影像图上坐标（行列号）
            Mosaic_all_imgs,    # ✔将指定路径文件夹下的tif影像全部镶嵌到一张影像上
            Mosaic_2img_to_one, # 将两幅影像镶嵌至同一幅影像
            raster_overlap,     # 两个栅格数据集取重叠区或求交集（仅测试方形影像）
            crop_tif_with_json_zero, # ✔将带有坐标的图像按照json矢量进行裁切,无数据区域为0
            crop_tif_with_json_nan,  # ✔将带有坐标的图像按照json矢量进行裁切,无数据区域为nan
            Merge_multi_json,   # ✔将多个小的json合并为一个大的json,
            resample_image,      # ✔使用GDAL对图像进行重采样
            shp_to_geojson,      # ✔将shp文件转为geojson文件
            clip_big_image, # 将大图裁切为小图，支持重叠
        )

---
示例程序：
----       
示例1-根据json批量裁切影像根据矢量批量裁切影像

    >>> from ATL_Tools import mkdir_or_exist, find_data_list
    >>> from ATL_Tools.ATL_gdal import crop_tif_with_json_nan
    >>> from tqdm import tqdm
    >>> import os 

    >>> img_path_all = '../推理出的结果_24类_RGB/'
    >>> output_path_all = '../推理出的结果_24类_RGB_crop'
    >>> json_path_all = '../要推理的json/'
    >>> mkdir_or_exist(output_path_all)
    >>> img_list = find_data_list(img_path_all, suffix='.tif')


    >>> for img_path in tqdm(img_list, colour='Green'):

    >>>     img_output_path = os.path.join(output_path_all, os.path.basename(img_path))
    >>>     json_path = os.path.join(json_path_all, os.path.basename(img_path).split('_')[-1].replace('.tif', '.json'))
    >>>     print(f'正在裁切: {img_output_path},json: {json_path}')
    >>>     crop_tif_with_json_nan(img_path, img_output_path, json_path) 

"""
from .ATL_path import *
# from .ATL_gdal import * # 为了可以不安装gdal使用 `conda install gdal`
